import asyncdispatch
import ../blobsets, ./filestores, ./httpstores
import spryvm/spryvm
import cbor
import std/strutils, std/tables, std/os

type BlobIdSpryNode* = ref object of Value
  id*: BlobId

type BlobSetSpryNode* = ref object of Value
  set*: BlobSet

method eval*(self: BlobIdSpryNode|BlobSetSpryNode; spry: Interpreter): Node = self

method `$`*(self: BlobIdSpryNode): string = $self.id

method `$`*(self: BlobSetSpryNode): string = "<BlobSet>"

type BlobStoreNode* = ref object of Value
  ## Object for store interface and caches
  store: BlobStore
  blobs: Table[string, tuple[id: BlobId, size: BiggestInt]]
  sets: Table[string, BlobSet]

proc insertPath(store: BlobStore; bs: BlobSet; kind: PathComponent; path: string): BlobSet =
  result = bs
  try:
    case kind
    of pcFile, pcLinkToFile:
      var path = normalizedPath path
      let (id, size) = waitFor store.ingestFile(path)
      path.removePrefix(getCurrentDir())
      path.removePrefix("/")
      result = result.insert(path, id, size)
      writeLine(stdout, id, align($size, 11), " ", path)
    of pcDir, pcLinkToDir:
      for kind, subPath in path.walkDir:
        result = store.insertPath(result, kind, subPath)
  except:
    let e = getCurrentException()
    writeLine(stderr, "failed to ingest '", path, "', ", e.msg)

proc getSet(env: BlobStoreNode; path: string): BlobSet =
  result = env.sets.getOrDefault(path)
  if result.isNil:
    result = newBlobSet()
    result = env.store.insertPath(result, path.getFileInfo.kind, path)
    if not result.isEmpty:
      env.sets[path] = result

proc ingestBlobs(env: BlobStoreNode; path: string): BlobSetSpryNode =
  let bs = env.getSet(path)
  BlobSetSpryNode(set: bs)

# Spry Blobset module
proc addBlobSets*(spry: Interpreter) =

  nimMeth "ingestBlobs":
    var env = BlobStoreNode(evalArgInfix(spry))
    let path = StringVal(evalArg(spry)).value
    result = ingestBlobs(env, path)

  nimFunc "newBlobStore":
    proc newStore(s: string): BlobStore =
        if s.startsWith "http://": newHttpStore(s)
        else: newFileStore(s)
    let arg = StringVal(evalArg(spry)).value
    BlobStoreNode(
      store: newStore(arg),
      blobs: initTable[string, tuple[id: BlobId, size: BiggestInt]](),
      sets: initTable[string, BlobSet]())

  nimFunc "toBlobId":
    let str = StringVal(evalArg(spry)).value
    BlobIdSpryNode(id: str.toBlobId)

  nimFunc "cbor":
    let set = BlobSetSpryNode(evalArg(spry)).set
    newValue($toCbor(set))

  nimFunc "commit":
    let
      env = BlobStoreNode(evalArgInfix(spry))
      set = BlobSetSpryNode(evalArg(spry)).set
    BlobSetSpryNode(set: env.store.commit(set))

  nimFunc "hex":
    let id = BlobIdSpryNode(evalArg(spry)).id
    newValue(id.toHex)

  #nimFunc "key":
  #  let str = StringVal(evalArg(spry)).value
  #  newValue(str.toKey)
